@@ -20,6 +20,8 @@
  */
 
 #include <gsf-config.h>
+#include <string.h>
+
 #include <gsf-gnome/gsf-input-gnomevfs.h>
 #include <gsf/gsf-input-memory.h>
 #include <gsf/gsf-input-impl.h>
@@ -53,9 +55,10 @@ gsf_input_gnomevfs_new_uri (GnomeVFSURI *uri, GError **error)
 	GnomeVFSResult    res;
 	GnomeVFSFileType  type;
 	gsf_off_t	  size;
+	gboolean          is_local;
 
 	if (uri == NULL) {
-		g_set_error (error, gsf_input_error (), 0,
+		g_set_error (error, gsf_input_error_id (), 0,
 			     "Filename/URI cannot be NULL");
 		return NULL;
 	}
@@ -64,16 +67,18 @@ gsf_input_gnomevfs_new_uri (GnomeVFSURI *uri, GError **error)
 		goto make_local_copy;
 
 	info = gnome_vfs_file_info_new ();
-	res = gnome_vfs_get_file_info_uri (uri, info, GNOME_VFS_FILE_INFO_DEFAULT);
-	size = (gsf_off_t)info->size ;
+	res = gnome_vfs_get_file_info_uri (uri, info, GNOME_VFS_FILE_INFO_DEFAULT | GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+
+	size = (gsf_off_t)info->size;
 	type = info->type;
+	is_local = GNOME_VFS_FILE_INFO_LOCAL (info);
 	gnome_vfs_file_info_unref (info);
 
 	switch (res) {
 	case GNOME_VFS_ERROR_NOT_SUPPORTED:
 		goto make_local_copy;
 	default:
-		g_set_error (error, gsf_input_error (), (gint) res,
+		g_set_error (error, gsf_input_error_id (), (gint) res,
 			     gnome_vfs_result_to_string (res));
 		return NULL;
 	case GNOME_VFS_OK: /* Nothing */ ;
@@ -92,15 +97,19 @@ gsf_input_gnomevfs_new_uri (GnomeVFSURI *uri, GError **error)
 			goto make_local_copy;
 		}
 
-		g_set_error (error, gsf_input_error (), 0,
+		g_set_error (error, gsf_input_error_id (), 0,
 			     "Not a regular file");
 		return NULL;
 	}
 
+	/* Make copies of small files.  */
+	if (!is_local && size < (256 << 10))
+		goto make_local_copy;
+
 	res = gnome_vfs_open_uri (&handle, uri,
 				  GNOME_VFS_OPEN_READ | GNOME_VFS_OPEN_RANDOM);
 	if (res != GNOME_VFS_OK) {
-		g_set_error (error, gsf_input_error (), (gint) res,
+		g_set_error (error, gsf_input_error_id (), (gint) res,
 			     gnome_vfs_result_to_string (res));
 		return NULL;
 	}
@@ -131,14 +140,14 @@ gsf_input_gnomevfs_new_uri (GnomeVFSURI *uri, GError **error)
 		res = gnome_vfs_read_entire_file (uri_text, &file_size, &buffer);
 		g_free (uri_text);
 		if (res != GNOME_VFS_OK) {
-			g_set_error (error, gsf_input_error (), (gint)res,
+			g_set_error (error, gsf_input_error_id (), (gint)res,
 				     "Read error while creating local copy.");
 			return NULL;
 		}
 
 		mem = gsf_input_memory_new (buffer, file_size, TRUE);
 		if (!mem) {
-			g_set_error (error, gsf_input_error (), 0,
+			g_set_error (error, gsf_input_error_id (), 0,
 				     "Failed to create local memory stream");
 			g_free (buffer);
 			return NULL;
@@ -164,7 +173,7 @@ gsf_input_gnomevfs_new (char const *text_uri, GError **error)
 {
 	GnomeVFSURI *uri = gnome_vfs_uri_new (text_uri);
 	if (!uri) {
-		g_set_error (error, gsf_input_error (), 0,
+		g_set_error (error, gsf_input_error_id (), 0,
 			     "Invalid URI");
 		return NULL;
 	} else {
@@ -189,11 +198,9 @@ gsf_input_gnomevfs_finalize (GObject *obj)
 		input->uri = NULL;
 	}
 
-	if (input->buf != NULL) {
-		g_free (input->buf);
-		input->buf  = NULL;
-		input->buf_size = 0;
-	}
+	g_free (input->buf);
+	input->buf = NULL;
+	input->buf_size = 0;
 
 	parent_class = g_type_class_peek (GSF_INPUT_TYPE);
 	if (parent_class && parent_class->finalize)
@@ -221,8 +228,7 @@ gsf_input_gnomevfs_read (GsfInput *input, size_t num_bytes,
 	if (buffer == NULL) {
 		if (vfs->buf_size < num_bytes) {
 			vfs->buf_size = num_bytes;
-			if (vfs->buf != NULL)
-				g_free (vfs->buf);
+			g_free (vfs->buf);
 			vfs->buf = g_new (guint8, vfs->buf_size);
 		}
 		buffer = vfs->buf;
@@ -245,16 +251,25 @@ static gboolean
 gsf_input_gnomevfs_seek (GsfInput *input, gsf_off_t offset, GSeekType whence)
 {
 	GsfInputGnomeVFS const *vfs     = GSF_INPUT_GNOMEVFS (input);
-	GnomeVFSSeekPosition vfs_whence = 0; /* make compiler shut up */    
+	GnomeVFSSeekPosition vfs_whence;
 
 	if (vfs->handle == NULL)
 		return TRUE;
 
 	switch (whence) {
+	default:
 	case G_SEEK_SET : vfs_whence = GNOME_VFS_SEEK_START;	break;
 	case G_SEEK_CUR : vfs_whence = GNOME_VFS_SEEK_CURRENT;	break;
 	case G_SEEK_END : vfs_whence = GNOME_VFS_SEEK_END;	break;
-	default : break;
+	}
+
+	/* Work around http://bugzilla.gnome.org/show_bug.cgi?id=152844  */
+	if (whence == G_SEEK_SET && offset > 0 && offset == gsf_input_size (input)) {
+		if (gsf_input_gnomevfs_seek (input, offset - 1, whence))
+			return TRUE;
+		if (gsf_input_gnomevfs_read (input, 1, NULL) == NULL)
+			return TRUE;
+		return FALSE;
 	}
 
 	if (GNOME_VFS_OK == gnome_vfs_seek (vfs->handle,vfs_whence,
