@@ -2,7 +2,7 @@
 /*
  * gsf-impl-utils.h: 
  *
- * Copyright (C) 2002-2003 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2002-2004 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -27,6 +27,15 @@
 
 G_BEGIN_DECLS
 
+/* We need to do this with a version check as this header gets installed.  */
+#if GLIB_CHECK_VERSION(2,7,0)
+#define GSF_PARAM_STATIC (G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB)
+#else
+#define GSF_PARAM_STATIC 0
+#endif
+
+/*************************************************************************/
+
 #define	GSF_CLASS_FULL(name, prefix, class_init, instance_init, parent_type, \
 		       abstract, interface_decl) \
 GType									\
@@ -60,41 +69,70 @@ prefix ## _get_type (void)						\
 	GSF_CLASS_FULL(name, prefix, class_init, instance_init, parent, \
 		       G_TYPE_FLAG_ABSTRACT, {})
 
-#define	GSF_DYNAMIC_CLASS_FULL(name, prefix, class_init, instance_init, parent_type, \
-			       abstract, interface_decl, plugin, type) \
-	if (type == 0) {						\
-		static GTypeInfo const type_info = {			\
-			sizeof (name ## Class),				\
-			(GBaseInitFunc) NULL,				\
-			(GBaseFinalizeFunc) NULL,			\
-			(GClassInitFunc) class_init,			\
-			(GClassFinalizeFunc) NULL,			\
-			NULL,	/* class_data */			\
-			sizeof (name),					\
-			0,	/* n_preallocs */			\
-			(GInstanceInitFunc) instance_init,		\
-			NULL						\
-		};							\
-		type = g_type_module_register_type (plugin, parent_type, #name,	\
-			&type_info, (GTypeFlags) abstract);		\
-		interface_decl						\
-	}
-
-#define	GSF_DYNAMIC_CLASS(name, prefix, class_init, instance_init, parent, plugin, type) \
-	GSF_DYNAMIC_CLASS_FULL(name, prefix, class_init, instance_init, parent, \
-			       0, {}, plugin, type)
-#define	GSF_DYNAMIC_CLASS_ABSTRACT(name, prefix, class_init, instance_init, parent, plugin, type) \
-	GSF_DYNAMIC_CLASS_FULL(name, prefix, class_init, instance_init, parent, \
-		       G_TYPE_FLAG_ABSTRACT, {}, plugin, type)
-
 #define GSF_INTERFACE_FULL(type, init_func, iface_type) {	\
 	static GInterfaceInfo const iface = {			\
 		(GInterfaceInitFunc) init_func, NULL, NULL };	\
 	g_type_add_interface_static (type, iface_type, &iface);	\
 }
+
 #define GSF_INTERFACE(init_func, iface_type)			\
 	GSF_INTERFACE_FULL(type, init_func, iface_type)
 
+/*************************************************************************/
+
+#define	GSF_DYNAMIC_CLASS_FULL(name, prefix, class_init, instance_init, parent_type, \
+			       abstract, interface_decl) 		\
+static GType prefix ## _type; 						\
+									\
+GType prefix ## _get_type (void);					\
+void  prefix ## _register_type (GTypeModule *module);			\
+									\
+GType									\
+prefix ## _get_type ()							\
+{									\
+	g_return_val_if_fail (prefix ## _type != 0, 0); 		\
+	return prefix ## _type;						\
+}									\
+void									\
+prefix ## _register_type (GTypeModule *module)				\
+{									\
+	static GTypeInfo const type_info = {				\
+		sizeof (name ## Class),					\
+		(GBaseInitFunc) NULL,					\
+		(GBaseFinalizeFunc) NULL,				\
+		(GClassInitFunc) class_init,				\
+		(GClassFinalizeFunc) NULL,				\
+		NULL,	/* class_data */				\
+		sizeof (name),						\
+		0,	/* n_preallocs */				\
+		(GInstanceInitFunc) instance_init,			\
+		NULL							\
+	};								\
+	GType type;							\
+									\
+	g_return_if_fail (prefix ## _type == 0); 			\
+									\
+	type = prefix ## _type = g_type_module_register_type (module,	\
+		parent_type, #name, &type_info, (GTypeFlags) abstract);	\
+	interface_decl							\
+}
+
+#define	GSF_DYNAMIC_CLASS(name, prefix, class_init, instance_init, parent)	\
+	GSF_DYNAMIC_CLASS_FULL(name, prefix, class_init, instance_init, parent,	\
+			       0, {})
+#define	GSF_DYNAMIC_CLASS_ABSTRACT(name, prefix, class_init, instance_init, parent) \
+	GSF_DYNAMIC_CLASS_FULL(name, prefix, class_init, instance_init, parent, \
+		       G_TYPE_FLAG_ABSTRACT, {})
+
+#define GSF_DYNAMIC_INTERFACE_FULL(type, init_func, iface_type, module) {	\
+	static GInterfaceInfo const iface = {					\
+		(GInterfaceInitFunc) init_func, NULL, NULL };			\
+	g_type_module_add_interface (module, type, iface_type, &iface);		\
+}
+
+#define GSF_DYNAMIC_INTERFACE(init_func, iface_type, module)			\
+	GSF_DYNAMIC_INTERFACE_FULL(type, init_func, iface_type, module)
+
 G_END_DECLS
 
 #endif /* GSF_IMPL_UTILS_H */
