@@ -19,12 +19,17 @@
  * USA
  */
 
+/* Lots of useful information in
+ *   http://www.aafassociation.org/html/specs/aafcontainerspec-v1.0.1.pdf
+ */
+
 #include <gsf-config.h>
 #include <gsf/gsf-infile-impl.h>
 #include <gsf/gsf-infile-msole.h>
 #include <gsf/gsf-impl-utils.h>
 #include <gsf/gsf-utils.h>
 #include <gsf/gsf-msole-impl.h>
+#include <gsf/gsf-input-proxy.h>
 
 #include <string.h>
 #include <stdio.h>
@@ -234,6 +239,8 @@ ole_info_get_sb_file (GsfInfileMSOle *parent)
 
 	parent->info->sb_file = gsf_infile_msole_new_child (parent,
 		parent->info->root_dir, NULL);
+	if (!parent->info->sb_file)
+		return NULL;
 
 	/* avoid creating a circular reference */
 	ole_info_unref (((GsfInfileMSOle *)parent->info->sb_file)->info);
@@ -437,8 +444,12 @@ ole_dup (GsfInfileMSOle const *src, GError **err)
 	g_return_val_if_fail (src != NULL, NULL);
 
 	input = gsf_input_dup (src->input, err);
-	if (input == NULL)
+	if (input == NULL) {
+		if (err != NULL)
+			*err = g_error_new (gsf_input_error_id (), 0,
+					    "Failed to duplicate input stream");
 		return NULL;
+	}
 
 	dst = (GsfInfileMSOle *)g_object_new (GSF_INFILE_MSOLE_TYPE, NULL);
 	dst->input = input;
@@ -471,11 +482,11 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	guint32 metabat_block, *ptr;
 
 	/* check the header */
-	if (gsf_input_seek (ole->input, (gsf_off_t) 0, G_SEEK_SET) ||
+	if (gsf_input_seek (ole->input, 0, G_SEEK_SET) ||
 	    NULL == (header = gsf_input_read (ole->input, OLE_HEADER_SIZE, NULL)) ||
 	    0 != memcmp (header, signature, sizeof (signature))) {
 		if (err != NULL)
-			*err = g_error_new (gsf_input_error (), 0,
+			*err = g_error_new (gsf_input_error_id (), 0,
 				"No OLE2 signature");
 		return TRUE;
 	}
@@ -494,7 +505,7 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	 */
 	if (6 > bb_shift || bb_shift >= 31 || sb_shift > bb_shift) {
 		if (err != NULL)
-			*err = g_error_new (gsf_input_error (), 0,
+			*err = g_error_new (gsf_input_error_id (), 0,
 				"Unreasonable block sizes");
 		return TRUE;
 	}
@@ -569,7 +580,7 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 
 	if (ptr == NULL) {
 		if (err != NULL)
-			*err = g_error_new (gsf_input_error (), 0,
+			*err = g_error_new (gsf_input_error_id (), 0,
 				"Inconsistent block allocation table");
 		return TRUE;
 	}
@@ -577,7 +588,7 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	/* Read the directory's bat, we do not know the size */
 	if (ole_make_bat (&info->bb.bat, 0, dirent_start, &ole->bat)) {
 		if (err != NULL)
-			*err = g_error_new (gsf_input_error (), 0,
+			*err = g_error_new (gsf_input_error_id (), 0,
 				"Problems making block allocation table");
 		return TRUE;
 	}
@@ -586,7 +597,7 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	ole->dirent = info->root_dir = ole_dirent_new (ole, 0, NULL);
 	if (ole->dirent == NULL) {
 		if (err != NULL)
-			*err = g_error_new (gsf_input_error (), 0,
+			*err = g_error_new (gsf_input_error_id (), 0,
 				"Problems reading directory");
 		return TRUE;
 	}
@@ -727,6 +738,9 @@ gsf_infile_msole_new_child (GsfInfileMSOle *parent,
 	size_t size_guess;
 
 	child = ole_dup (parent, err);
+	if (!child)
+		return NULL;
+
 	child->dirent = dirent;
 	gsf_input_set_size (GSF_INPUT (child), (gsf_off_t) dirent->size);
 
@@ -750,6 +764,13 @@ gsf_infile_msole_new_child (GsfInfileMSOle *parent,
 		metabat = &info->sb.bat;
 		size_guess = dirent->size >> info->sb.shift;
 		sb_file = ole_info_get_sb_file (parent);
+		if (!sb_file) {
+			if (err != NULL)
+				*err = g_error_new (gsf_input_error_id (), 0,
+						    "Failed to access child");
+			g_object_unref (G_OBJECT (child));
+			return NULL;
+		}
 	} else {
 		metabat = &info->bb.bat;
 		size_guess = dirent->size >> info->bb.shift;
@@ -891,13 +912,14 @@ gsf_infile_msole_new (GsfInput *source, GError **err)
 	g_return_val_if_fail (GSF_IS_INPUT (source), NULL);
 
 	ole = (GsfInfileMSOle *)g_object_new (GSF_INFILE_MSOLE_TYPE, NULL);
-	g_object_ref (G_OBJECT (source));
-	ole->input = source;
-	gsf_input_set_size (GSF_INPUT (ole), (gsf_off_t) 0);
+	ole->input = gsf_input_proxy_new (source);
+	gsf_input_set_size (GSF_INPUT (ole), 0);
 
 	calling_pos = gsf_input_tell (source);
 	if (ole_init_info (ole, err)) {
-		gsf_input_seek (source, calling_pos, G_SEEK_SET);
+		/* It's not clear to me why we do this.  And if this
+		   fails, there's really nothing we can do.  */
+		(void)gsf_input_seek (source, calling_pos, G_SEEK_SET);
 
 		g_object_unref (G_OBJECT (ole));
 		return NULL;
