@@ -128,30 +128,35 @@ gsf_input_class_init (GObjectClass *gobject_class)
 					 g_param_spec_string ("name", "Name",
 							      "The Input's Name",
 							      NULL,
+							      GSF_PARAM_STATIC |
 							      G_PARAM_READABLE));
 	g_object_class_install_property (gobject_class,
 					 PROP_SIZE,
 					 g_param_spec_int64 ("size", "Size",
 							     "The Input's Size",
 							     0, G_MAXINT64, 0,
+							     GSF_PARAM_STATIC |
 							     G_PARAM_READABLE));
 	g_object_class_install_property (gobject_class,
 					 PROP_EOF,
 					 g_param_spec_boolean ("eof", "OEF",
 							       "End Of File",
 							       FALSE,
+							       GSF_PARAM_STATIC |
 							       G_PARAM_READABLE));
 	g_object_class_install_property (gobject_class,
 					 PROP_REMAINING,
 					 g_param_spec_int64 ("remaining", "Remaining",
 							     "Amount of Data Remaining",
 							     0, G_MAXINT64, 0,
+							     GSF_PARAM_STATIC |
 							     G_PARAM_READABLE));
 	g_object_class_install_property (gobject_class,
 					 PROP_POS,
 					 g_param_spec_int64 ("position", "Position",
 							     "The Output's Current Position",
 							     0, G_MAXINT64, 0,
+							     GSF_PARAM_STATIC |
 							     G_PARAM_READABLE));
 }
 
@@ -161,9 +166,9 @@ GSF_CLASS_ABSTRACT (GsfInput, gsf_input,
 
 /**
  * gsf_input_name :
- * @input : 
+ * @input : the input stream
  *
- * Returns @input's name in utf8 form, DO NOT FREE THIS STRING
+ * Returns @input's name in utf8 form, or NULL if it has no name.
  **/
 char const *
 gsf_input_name (GsfInput *input)
@@ -174,7 +179,7 @@ gsf_input_name (GsfInput *input)
 
 /**
  * gsf_input_container :
- * @input : 
+ * @input : the input stream
  *
  * Returns, but does not add a reference to @input's container.
  * Potentially NULL
@@ -204,17 +209,51 @@ gsf_input_dup (GsfInput *input, GError **err)
 
 	dst = GET_CLASS (input)->Dup (input, err);
 	if (dst != NULL) {
-		dst->size = input->size;
+		if (dst->size != input->size) {
+			if (err != NULL)
+				*err = g_error_new (gsf_input_error_id (), 0,
+						    "Duplicate size mismatch");
+			g_object_unref (dst);
+			return NULL;
+		}
+		if (gsf_input_seek (dst, input->cur_offset, G_SEEK_SET)) {
+			if (err != NULL)
+				*err = g_error_new (gsf_input_error_id (), 0,
+						    "Seek failed");
+			g_object_unref (dst);
+			return NULL;
+		}
+
 		if (input->name != NULL)
 			gsf_input_set_name (dst, input->name);
 		dst->container = input->container;
 		if (dst->container != NULL)
 			g_object_ref (G_OBJECT (dst->container));
-		gsf_input_seek (dst, (gsf_off_t)input->cur_offset, G_SEEK_SET);
 	}
 	return dst;
 }
 
+/**
+ * gsf_input_open_sibling :
+ * @input : The input
+ *
+ * NOT CURRENTLY IMPLEMENTED
+ *
+ * Attempts to open a 'sibling' of @input.  The caller is responsible for
+ * managing the resulting object.
+ *
+ * NOT CURRENTLY IMPLEMENTED
+ *
+ * Returns :  the size or -1 on error
+ **/
+GsfInput *
+gsf_input_sibling (GsfInput const *input, char const *name, GError **err)
+{
+	g_return_val_if_fail (GET_CLASS (input)->OpenSibling, NULL);
+
+	return GET_CLASS (input)->OpenSibling (input, name, err);
+}
+
 /**
  * gsf_input_size :
  * @input : The input
@@ -248,11 +287,11 @@ gsf_input_eof (GsfInput *input)
 
 /**
  * gsf_input_read :
- * @input :
- * @num_bytes :
- * @optional_buffer : If supplied copy the data into it
+ * @input : the input stream
+ * @num_bytes : number of bytes to read
+ * @optional_buffer : NULL, or pointer to destination memory area
  *
- * Read at at least @num_bytes.  Does not change the current position if there
+ * Read at least @num_bytes.  Does not change the current position if there
  * is an error.  Will only read if the entire amount can be read.  Invalidates
  * the buffer associated with previous calls to gsf_input_read.
  *
@@ -274,12 +313,11 @@ gsf_input_read (GsfInput *input, size_t num_bytes, guint8 *optional_buffer)
 
 	input->cur_offset += num_bytes;
 	return res;
-
 }
 
 /**
  * gsf_input_remaining :
- * @input :
+ * @input : the input stream
  *
  * Returns the number of bytes left in the file.
  **/
@@ -293,7 +331,7 @@ gsf_input_remaining (GsfInput *input)
 
 /**
  * gsf_input_tell :
- * @input :
+ * @input : the input stream
  *
  * Returns the current offset in the file.
  **/
@@ -307,9 +345,10 @@ gsf_input_tell (GsfInput *input)
 
 /**
  * gsf_input_seek :
- * @input :
- * @offset :
- * @whence :
+ * @input : the input stream
+ * @offset : target offset
+ * @whence : determines whether the offset is relative to the beginning or
+ *           the end of the stream, or to the current location.
  *
  * Returns TRUE on error.
  **/
@@ -339,14 +378,15 @@ gsf_input_seek (GsfInput *input, gsf_off_t offset, GSeekType whence)
 
 	if (GET_CLASS (input)->Seek (input, offset, whence))
 		return TRUE;
+
 	input->cur_offset = pos;
 	return FALSE;
 }
 
 /**
  * gsf_input_set_name :
- * @input :
- * @name :
+ * @input : the input stream
+ * @name : the new name of the stream, or NULL.
  *
  * protected.
  *
@@ -360,15 +400,34 @@ gsf_input_set_name (GsfInput *input, char const *name)
 	g_return_val_if_fail (input != NULL, FALSE);
 
 	buf = g_strdup (name);
-	if (input->name != NULL)
-		g_free (input->name);
+	g_free (input->name);
 	input->name = buf;
 	return TRUE;
 }
 
+/**
+ * gsf_input_set_name_from_filename :
+ * @input : the input stream
+ * @filename : the (fs-sys encoded) filename
+ *
+ * protected.
+ *
+ * Returns : TRUE if the assignment was ok.
+ **/
+gboolean
+gsf_input_set_name_from_filename (GsfInput *input, char const *filename)
+{
+	g_return_val_if_fail (input != NULL, FALSE);
+
+	g_free (input->name);
+	input->name = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
+	return TRUE;
+}
+
+
 /**
  * gsf_input_set_container :
- * @input :
+ * @input : the input stream
  * @container :
  *
  * Returns : TRUE if the assignment was ok.
@@ -388,8 +447,8 @@ gsf_input_set_container (GsfInput *input, GsfInfile *container)
 
 /**
  * gsf_input_set_size :
- * @input :
- * @size :
+ * @input : the input stream
+ * @size : the size of the stream
  *
  * Returns : TRUE if the assignment was ok.
  */
@@ -397,6 +456,7 @@ gboolean
 gsf_input_set_size (GsfInput *input, gsf_off_t size)
 {
 	g_return_val_if_fail (input != NULL, FALSE);
+	g_return_val_if_fail (size >= 0, FALSE);
 
 	input->size = size;
 	return TRUE;
@@ -404,10 +464,10 @@ gsf_input_set_size (GsfInput *input, gsf_off_t size)
 
 /**
  * gsf_input_seek_emulate: Emulate forward seeks by reading.
- * @input :
- * @pos :
+ * @input : stream to emulate seek for
+ * @pos : absolute position to seek to
  *
- * Returns : TRUE if the emulation worked.
+ * Returns : TRUE if the emulation failed.
  */
 gboolean
 gsf_input_seek_emulate (GsfInput *input, gsf_off_t pos)
@@ -425,18 +485,30 @@ gsf_input_seek_emulate (GsfInput *input, gsf_off_t pos)
 
 /****************************************************************************/
 
+/**
+ * gsf_input_error_id :
+ *
+ * Returns : A utility quark to flag a GError as being an input problem.
+ */
+GQuark 
+gsf_input_error_id (void)
+{
+	static GQuark quark;
+	if (!quark)
+		quark = g_quark_from_static_string ("gsf_input_error_id");
+	return quark;
+}
+
 /**
  * gsf_input_error :
  *
+ * Deprecated in 1.12.0
  * Returns : A utility quark to flag a GError as being an input problem.
  */
 GQuark 
 gsf_input_error (void)
 {
-	static GQuark quark;
-	if (!quark)
-		quark = g_quark_from_static_string ("gsf_input_error");
-	return quark;
+	return gsf_input_error_id ();
 }
 
 /****************************************************************************/
@@ -495,7 +567,7 @@ gsf_input_uncompress (GsfInput *src)
 	gsf_off_t cur_offset = src->cur_offset;
 	const guint8 *data;
 
-	if (gsf_input_seek (src, (gsf_off_t) 0, G_SEEK_SET))
+	if (gsf_input_seek (src, 0, G_SEEK_SET))
 		goto error;
 
 	/* Read header up front, so we avoid extra seeks in tests.  */
