@@ -620,6 +620,7 @@ gsf_xml_in_parse (GsfXMLIn *state, GsfInput *input)
 	gboolean res;
 
 	g_return_val_if_fail (state != NULL, FALSE);
+	g_return_val_if_fail (state->doc != NULL, FALSE);
 	g_return_val_if_fail (GSF_IS_INPUT (input), FALSE);
 
 	ctxt = gsf_xml_parser_context_full (input, &gsfXMLInParser, state);
@@ -1053,24 +1054,24 @@ gsf_xml_out_add_uint (GsfXMLOut *xml, char const *id,
  * @xml :
  * @id  : optionally NULL for content
  * @val : the value
- * @precision : the number of decimal points to display
+ * @precision : the number of significant digits to use, -1 meaning "enough".
  *
  * dump float value @val to an attribute named @id or as the nodes
- * content with precision @precision.
+ * content with precision @precision.  The number will be formattted
+ * according to the "C" locale.
  **/
 void
 gsf_xml_out_add_float (GsfXMLOut *xml, char const *id,
 		       double val, int precision)
 {
-	char buf [101 + DBL_DIG];
+	char format_str[4 * sizeof (int) + 10];
+	char buf[G_ASCII_DTOSTR_BUF_SIZE + DBL_DIG];
 
 	if (precision < 0 || precision > DBL_DIG)
 		precision = DBL_DIG;
 
-	if (fabs (val) < 1e9 && fabs (val) > 1e-5)
-		snprintf (buf, sizeof buf-1, "%.*g", precision, val);
-	else
-		snprintf (buf, sizeof buf-1, "%f", val);
+	sprintf (format_str, "%%.%dg", precision);
+	g_ascii_formatd (buf, sizeof (buf), format_str, val);
 	gsf_xml_out_add_cstr_unchecked (xml, id, buf);
 }
 
@@ -1093,6 +1094,19 @@ gsf_xml_out_add_color (GsfXMLOut *xml, char const *id,
 	gsf_xml_out_add_cstr_unchecked (xml, id, buf);
 }
 
+void
+gsf_xml_out_add_enum (GsfXMLOut *xml, char const *id, GType etype, gint val)
+{
+	GEnumClass *eclass = G_ENUM_CLASS (g_type_class_peek (etype));
+	GEnumValue *ev = g_enum_get_value (eclass, val);
+
+	if (ev)
+		gsf_xml_out_add_cstr_unchecked (xml, id, ev->value_name);
+	else
+		g_warning ("Invalid value %d for type %s",
+			   val, g_type_name (etype));
+}
+
 /**
  * gsf_xml_out_add_base64 :
  * @xml :
