@@ -212,7 +212,7 @@ msole_vt_name (GsfMSOleVariantType type)
 		"VT_BLOB",	"VT_STREAM",	"VT_STORAGE",	"VT_STREAMED_OBJECT",
 		"VT_STORED_OBJECT", "VT_BLOB_OBJECT", "VT_CF",	"VT_CLSID"
 	};
-	g_return_val_if_fail (type >= VT_EMPTY, "_UNKNOWN_");
+
 	type &= ~VT_VECTOR;
 	if (type <= VT_LPWSTR)
 		return names[type];
@@ -467,7 +467,7 @@ msole_prop_parse (GsfMSOleMetaDataSection *section,
 		/* 
 		 * This is the representation of many strings.  It is stored in
 		 * the same representation as VT_BSTR.  Note that the serialized
-		 * representation of VP_LPSTR has a preceding byte count, wheras
+		 * representation of VP_LPSTR has a preceding byte count, whereas
 		 * the in-memory representation does not.
 		 */
 		/* be anal and safe */
@@ -1167,7 +1167,7 @@ msole_metadata_write_section (WritePropState *state, gboolean user)
 	GsfDocProp const *prop;
 	gpointer tmp;
 	unsigned i;
-	GValue	 scratch = { 0 };
+	GValue	 scratch;
 
 	if (user && state->dict == NULL)
 		return TRUE;
@@ -1180,7 +1180,9 @@ msole_metadata_write_section (WritePropState *state, gboolean user)
 			     G_SEEK_END))
 		return FALSE;
 
+	memset (&scratch,  0, sizeof (GValue));
 	g_value_init (&scratch, G_TYPE_STRING);
+
 	offsets = g_alloca (sizeof (GsfMSOleMetaDataProp) * count);
 
 	/* 0) codepage */
@@ -1641,7 +1643,7 @@ gsf_msole_codepage_to_lid (int codepage)
  *
  * Returns our best guess at the codepage for the given language id
  **/
-guint
+int
 gsf_msole_lid_to_codepage (guint lid)
 {
 	if (lid == 0x0FFF) /* Macintosh Hack */
@@ -1861,7 +1863,7 @@ gsf_msole_lid_to_codepage_str (guint lid)
  * Returns our best guess at the applicable windows code page based on an
  * 	environment variable or the current locale.
  **/
-guint
+int
 gsf_msole_iconv_win_codepage (void)
 {
 	char *lang;
@@ -1886,7 +1888,7 @@ gsf_msole_iconv_win_codepage (void)
 }
 
 static GSList *
-gsf_msole_iconv_get_codepage_string_list (guint codepage)
+gsf_msole_iconv_get_codepage_string_list (int codepage)
 {
 	GSList *cp_list = NULL;
 
@@ -1902,6 +1904,7 @@ gsf_msole_iconv_get_codepage_string_list (guint codepage)
 			cp_list = g_slist_prepend (cp_list, g_strdup ("MACROMAN"));
 			cp_list = g_slist_prepend (cp_list, g_strdup ("MACINTOSH"));
 			break;
+		case -535:
 		case 65001:
 			cp_list = g_slist_prepend (cp_list, g_strdup ("UTF-8"));
 			break;
@@ -1938,7 +1941,7 @@ gsf_msole_iconv_open_codepage_for_import (char const *to, int codepage)
 	g_slist_free (codepage_list);
 
 	if (iconv_handle == (GIConv)(-1))
-		g_warning ("Unable to open an iconv handle from codepage %u -> %s",
+		g_warning ("Unable to open an iconv handle from codepage %d -> %s",
 			   codepage, to);
 	return iconv_handle;
 }
@@ -1966,7 +1969,7 @@ gsf_msole_iconv_open_for_import (int codepage)
  * 	windows codepage.
  **/
 GIConv
-gsf_msole_iconv_open_codepages_for_export (guint codepage_to, char const *from)
+gsf_msole_iconv_open_codepages_for_export (int codepage_to, char const *from)
 {
 	GIConv iconv_handle = (GIConv)(-1);
 	gchar *codepage_str;
@@ -1997,7 +2000,7 @@ gsf_msole_iconv_open_codepages_for_export (guint codepage_to, char const *from)
  * 	windows codepage.
  **/
 GIConv
-gsf_msole_iconv_open_codepage_for_export (guint codepage_to)
+gsf_msole_iconv_open_codepage_for_export (int codepage_to)
 {
 	return gsf_msole_iconv_open_codepages_for_export (codepage_to, "UTF-8");
 }
