@@ -539,6 +539,9 @@ gsf_outfile_msole_write (GsfOutput *output,
 	return TRUE;
 }
 
+static gsf_off_t gsf_outfile_msole_vprintf (GsfOutput *output,
+	char const *format, va_list args) G_GNUC_PRINTF (2, 0);
+
 static gsf_off_t
 gsf_outfile_msole_vprintf (GsfOutput *output, char const *format, va_list args)
 {
