@@ -2,7 +2,7 @@
 /*
  * gsf-output-stdio.c: stdio based output
  *
- * Copyright (C) 2002-2004 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2002-2005 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -24,6 +24,7 @@
 #include <gsf/gsf-output-impl.h>
 #include <gsf/gsf-impl-utils.h>
 #include <gsf/gsf-utils.h>
+#include <glib/gstdio.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -34,30 +35,19 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-
 #ifdef G_OS_WIN32
-
-#ifndef S_IRUSR
-#define S_IRUSR 04
-#define S_IWUSR 02
-#define W_OK    00  /* existance mode only */
-typedef int mode_t;
-#endif
-
-#define S_IRGRP S_IRUSR
-#define S_IROTH S_IRUSR
-#define S_ISGID 0
-
-#define getuid() 0
-#define getgid() 0
-#define chown(filename, uid, gid) 0
-
+#include <wchar.h>
+#include <direct.h>
+#include <glib/gwin32.h>
 #ifdef HAVE_IO_H
 #include <io.h>
 #endif
-
 #endif /* G_OS_WIN32 */
 
+#ifndef W_OK
+#define W_OK 2
+#endif
+
 static GObjectClass *parent_class;
 
 struct _GsfOutputStdio {
@@ -73,179 +63,136 @@ typedef struct {
 	GsfOutputClass output_class;
 } GsfOutputStdioClass;
 
-#define GSF_MAX_LINK_LEVEL 256
-#ifdef MAXPATHLEN
-#define GSF_MAX_PATH_LEN  MAXPATHLEN
-#elif defined (PATH_MAX)
-#define GSF_MAX_PATH_LEN  PATH_MAX
-#else
-#define GSF_MAX_PATH_LEN  2048
+static int
+rename_wrapper (char const *oldfilename, char const *newfilename)
+{
+	int result = g_rename (oldfilename, newfilename);
+#ifdef G_OS_WIN32
+	if (result) {
+		/* Win32's rename does not unlink the target.  */
+		(void)g_unlink (newfilename);
+		result = g_rename (oldfilename, newfilename);
+	}
 #endif
+	return result;
+}
 
-/* Does readlink() recursively until we find a real filename. */
+/* g_access would be nice here.  */
+static gboolean
+gsf_access (char const *filename, int what)
+{
+#ifdef G_OS_WIN32
+	int retval;
+	int save_errno;
+	if (G_WIN32_HAVE_WIDECHAR_API ()) {
+		wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
+		if (wfilename == NULL) {
+			errno = EINVAL;
+			return -1;
+		}
+
+		retval = _waccess (wfilename, what);
+		save_errno = errno;
+
+		g_free (wfilename);
+      
+		errno = save_errno;
+	} else {
+		gchar *cp_filename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
+		if (cp_filename == NULL) {
+			errno = EINVAL;
+			return -1;
+		}
+
+		retval = _access (cp_filename, what);
+		save_errno = errno;
+
+		g_free (cp_filename);
+
+		errno = save_errno;
+	}
+	return retval;
+#else
+	return access (filename, what);
+#endif
+}
+
+
+
+#define GSF_MAX_LINK_LEVEL 256
+
+/* Calls g_file_read_link() until we find a real filename. */
 static char *
 follow_symlinks (char const *filename, GError **error)
 {
-#ifdef HAVE_READLINK
-	gchar *followed_filename;
-	gint link_count;
+	gchar *followed_filename, *link;
+	gint link_count = 0;
 
 	g_return_val_if_fail (filename != NULL, NULL);
-	g_return_val_if_fail (strlen (filename) + 1 <= GSF_MAX_PATH_LEN, NULL);
 
 	followed_filename = g_strdup (filename);
-	link_count = 0;
 
-	while (link_count++ < GSF_MAX_LINK_LEVEL) {
-		char linkname[GSF_MAX_PATH_LEN];
-		int len = readlink (followed_filename, linkname, GSF_MAX_PATH_LEN - 1);
-
-		if (len == -1) {
-			switch (errno) {
-			case EINVAL: /* Not a symlink.  */
-			case ENOSYS: /* Surely not a symlink.  */
-			case ENOENT: /* No such file.  */
-				return followed_filename;
-
-			default:
-				if (error)
-					*error = g_error_new_literal 
-						(gsf_output_error_id (), errno,
-						 g_strerror (errno));
-				g_free (followed_filename);
-				return NULL;
-			}
-		}
-
-		linkname[len] = '\0';
-
-		if (g_path_is_absolute (linkname)) {
+	while ((link = g_file_read_link (followed_filename, NULL)) != NULL &&
+	  ++link_count <= GSF_MAX_LINK_LEVEL) {
+		if (g_path_is_absolute (link)) {
 			g_free (followed_filename);
-			followed_filename = g_strdup (linkname);
+			followed_filename = link;
 		} else {
-			/*
-			 * If the linkname is not an absolute path name, append
+			/* If the linkname is not an absolute path name, append
 			 * it to the directory name of the followed filename.  E.g.
 			 * we may have /foo/bar/baz.lnk -> eek.txt, which really
-			 * is /foo/bar/eek.txt.
-			 */
-
-			char *dir = g_path_get_dirname (followed_filename);
-
+			 * is /foo/bar/eek.txt.  */
+			gchar *dir = g_path_get_dirname (followed_filename);
 			g_free (followed_filename);
-			followed_filename = g_build_filename (dir, linkname, NULL);
+			followed_filename = g_build_filename (dir, link, NULL);
 			g_free (dir);
+			g_free (link);
 		}
 	}
 
+	if (link == NULL)
+		return followed_filename;
+
 	/* Too many symlinks */
-	if (error)
-		*error = g_error_new_literal (gsf_output_error_id (), ELOOP,
-					      g_strerror (ELOOP));
-
+	if (error != NULL) {
+#ifdef ELOOP
+		int err = ELOOP;
+#else
+		/* We have links, but not ELOOP.  Very strange.  */
+		int err = EINVAL;
+#endif
+		*error = g_error_new_literal (gsf_output_error_id (), err,
+					      g_strerror (err));
+	}
+	g_free (followed_filename);
 	return NULL;
-#else  /* !HAVE_READLINK */
-
-	return g_strdup (filename);
-
-#endif /* !HAVE_READLINK */
 }
 
-/**
- * gsf_output_stdio_new :
- * @filename : in utf8.
- * @err	     : optionally NULL.
- *
- * Returns a new file or NULL.
- **/
-GsfOutput *
-gsf_output_stdio_new (char const *filename, GError **err)
+static gboolean
+close_file_helper (GsfOutputStdio *stdio, gboolean seterr)
 {
-	GsfOutputStdio *stdio;
-	FILE *file = NULL;
-	char *dirname = NULL;
-	char *temp_filename = NULL;
-	char *real_filename = follow_symlinks (filename, err);
-	int fd;
-	mode_t saved_umask;
-	struct stat st;
+	gboolean res = (0 == fclose (stdio->file));
+	stdio->file = NULL;
+	if (!res && seterr)
+		gsf_output_set_error (GSF_OUTPUT (stdio), errno,
+				      "Failed to close file: %s",
+				      g_strerror (errno));
+	return res;
+}
 
-	if (real_filename == NULL)
-		goto failure;
+static gboolean
+unlink_file_helper (GsfOutputStdio *stdio)
+{
+	if (!stdio->temp_filename)
+		return TRUE;
 
-	/* Get the directory in which the real filename lives */
-	dirname = g_path_get_dirname (real_filename);
-
-	if (stat (real_filename, &st) == 0) {
-		/* FIXME: use eaccess if available.  */
-		/* FIXME? Race conditions en masse.  */
-		if (access (real_filename, W_OK) != 0) {
-			if (err != NULL)
-				*err = g_error_new_literal
-					(gsf_output_error_id (), errno,
-					 g_strerror (errno));
-			goto failure;
-		}
-	} else {
-		/*
-		 * File does not exist.  Compute the permissions and uid/gid
-		 * that we will use for the newly-created file.
-		 */
-
-		struct stat dir_st;
-		int result;
-
-		/* Use default permissions */
-		st.st_mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
-		st.st_uid = getuid ();
-
-		/* Used to set create_backup_copy = FALSE, but
-		   creating backup copies on this level is a 
-		   horrible mistake. -DAL- */
-
-		result = stat (dirname, &dir_st);
-
-		if (result == 0 && (dir_st.st_mode & S_ISGID))
-			st.st_gid = dir_st.st_gid;
-		else
-			st.st_gid = getgid ();
+	if (g_unlink (stdio->temp_filename) == 0) {
+		g_free (stdio->temp_filename);
+		stdio->temp_filename = NULL;
+		return TRUE;
 	}
 
-	/* Save to a temporary file.  We set the umask because some (buggy)
-	 * implementations of mkstemp() use permissions 0666 and we want 0600.
-	 */
-	temp_filename = g_build_filename (dirname, ".gsf-save-XXXXXX", NULL);
-	/* Oh, joy.  What about threads?  --MW */
-	saved_umask = umask (0077);
-	fd = g_mkstemp (temp_filename); /* this modifies temp_filename to the used name */
-	umask (saved_umask);
-
-	if (fd < 0 || NULL == (file = fdopen (fd, "wb"))) {
-		if (err != NULL)
-			*err = g_error_new_literal
-				(gsf_output_error_id (), errno,
-				 g_strerror (errno));
-		goto failure;
-	}
-
-	stdio = g_object_new (GSF_OUTPUT_STDIO_TYPE, NULL);
-	stdio->file = file;
-	stdio->st = st;
-	stdio->create_backup_copy = FALSE;
-	stdio->real_filename = real_filename;
-	stdio->temp_filename = temp_filename;
-
-	gsf_output_set_name_from_filename (GSF_OUTPUT (stdio), filename);
-
-	g_free (dirname);
-
-	return GSF_OUTPUT (stdio);
-
-failure :
-	g_free (temp_filename);
-	g_free (real_filename);
-	g_free (dirname);
-	return NULL;
+	return FALSE;
 }
 
 static gboolean
@@ -258,26 +205,33 @@ gsf_output_stdio_close (GsfOutput *output)
 	if (stdio->file == NULL)
 		return FALSE;
 
-	if (stdio->keep_open) {
-		stdio->file = NULL;
-		return TRUE;
-	}
+	if (gsf_output_error (output)) {
+		res = TRUE;
+		if (!stdio->keep_open && !close_file_helper (stdio, FALSE))
+			res = FALSE;
 
-	res = (0 == fclose (stdio->file));
-	stdio->file = NULL;
+		if (!unlink_file_helper (stdio))
+			res = FALSE;
 
-	/* short circuit our when dealing with raw FILE */
-	if (NULL == stdio->real_filename) {
-		if (!res)
-			gsf_output_set_error (output, errno,
-				"Failed to close file.");
 		return res;
 	}
 
+	if (stdio->keep_open) {
+		gboolean res = (0 == fflush (stdio->file));
+		if (!res)
+			gsf_output_set_error (output, errno,
+					      "Failed to flush.");
+		stdio->file = NULL;
+		return res;
+	}
+
+	res = close_file_helper (stdio, TRUE);
+
+	/* short circuit our when dealing with raw FILE */
+	if (!stdio->real_filename)
+		return res;
 	if (!res) {
-		gsf_output_set_error (output, errno,
-				      "Failed to close temporary file.");
-		unlink (stdio->temp_filename);
+		unlink_file_helper (stdio);
 		return FALSE;
 	}
 
@@ -285,24 +239,24 @@ gsf_output_stdio_close (GsfOutput *output)
 	if (stdio->create_backup_copy) {
 		gint result;
 		backup_filename = g_strconcat (stdio->real_filename, ".bak", NULL);
-		result = rename (stdio->real_filename, backup_filename);
+		result = rename_wrapper (stdio->real_filename, backup_filename);
 		if (result != 0) {
-			char *utf8name = gsf_filename_to_utf8 (backup_filename, TRUE);
+			char *utf8name = g_filename_display_name (backup_filename);
 			gsf_output_set_error (output, errno,
-				"Could not backup the original as %s.",
-				utf8name);
+					      "Could not backup the original as %s.",
+					      utf8name);
 			g_free (utf8name);
 			g_free (backup_filename);
-			unlink (stdio->temp_filename);
+			g_unlink (stdio->temp_filename);
 			return FALSE;
 		}
 	}
 
 	/* Move the temp file to the original file */
-	if (rename (stdio->temp_filename, stdio->real_filename) != 0) {
+	if (rename_wrapper (stdio->temp_filename, stdio->real_filename) != 0) {
 		gint saved_errno = errno;
 		if (backup_filename != NULL &&
-		    rename (backup_filename, stdio->real_filename) != 0)
+		    rename_wrapper (backup_filename, stdio->real_filename) != 0)
 			saved_errno = errno;
 		res = gsf_output_set_error (output, errno, g_strerror (errno));
 	} else {
@@ -310,7 +264,9 @@ gsf_output_stdio_close (GsfOutput *output)
 		 * can do here, I'm afraid.  The final data is saved anyways.
 		 * Note the order: mode, uid+gid, gid, uid, mode.
 		 */
+#warning "We need g_chmod in gstdio.h for this"
 		chmod (stdio->real_filename, stdio->st.st_mode);
+#ifdef HAVE_CHOWN
 		if (chown (stdio->real_filename,
 			   stdio->st.st_uid,
 			   stdio->st.st_gid)) {
@@ -319,6 +275,7 @@ gsf_output_stdio_close (GsfOutput *output)
 			chown (stdio->real_filename, stdio->st.st_uid, -1);
 		}
 		chmod (stdio->real_filename, stdio->st.st_mode);
+#endif
 	}
 
 	g_free (backup_filename);
@@ -410,6 +367,9 @@ gsf_output_stdio_write (GsfOutput *output,
 	return TRUE;
 }
 
+static gsf_off_t gsf_output_stdio_vprintf (GsfOutput *output,
+	char const *fmt, va_list args) G_GNUC_PRINTF (2, 0);
+
 static gsf_off_t
 gsf_output_stdio_vprintf (GsfOutput *output, char const *fmt, va_list args)
 {
@@ -443,15 +403,176 @@ gsf_output_stdio_class_init (GObjectClass *gobject_class)
 GSF_CLASS (GsfOutputStdio, gsf_output_stdio,
 	   gsf_output_stdio_class_init, gsf_output_stdio_init, GSF_OUTPUT_TYPE)
 
+GsfOutput *
+gsf_output_stdio_new_valist (char const *filename, GError **err,
+			     char const *first_property_name,
+			     va_list     var_args)
+{
+	GsfOutputStdio *stdio;
+	FILE *file = NULL;
+	char *dirname = NULL;
+	char *temp_filename = NULL;
+	char *real_filename = follow_symlinks (filename, err);
+	int fd;
+	mode_t saved_umask;
+	struct stat st;
+	gboolean fixup_mode = FALSE;
+
+	if (real_filename == NULL)
+		goto failure;
+
+	/* Get the directory in which the real filename lives */
+	dirname = g_path_get_dirname (real_filename);
+
+	if (g_stat (real_filename, &st) == 0) {
+		if (!S_ISREG (st.st_mode)) {
+			if (err != NULL) {
+				char *dname = g_filename_display_name
+					(real_filename);
+				*err = g_error_new (gsf_output_error_id (), 0,
+						    "%s: Is not a regular file",
+						    dname);
+				g_free (dname);
+			}
+			goto failure;
+		}
+
+		/* FIXME? Race conditions en masse.  */
+		if (gsf_access (real_filename, W_OK) == -1) {
+			if (err != NULL) {
+				int save_errno = errno;
+				char *dname = g_filename_display_name
+					(real_filename);
+				*err = g_error_new
+					(gsf_output_error_id (), errno,
+					 "%s: %s",
+					 dname, g_strerror (save_errno));
+				g_free (dname);
+			}
+			goto failure;
+		}
+	} else {
+		/*
+		 * File does not exist.  Compute the permissions and uid/gid
+		 * that we will use for the newly-created file.
+		 */
+
+		memset (&st, 0, sizeof (st));
+
+		/* Use default permissions */
+		st.st_mode = 0666;  fixup_mode = TRUE;
+#ifdef HAVE_CHOWN
+		{
+			struct stat dir_st;
+
+			st.st_uid = getuid ();
+
+			if (g_stat (dirname, &dir_st) == 0 &&
+			    S_ISDIR (dir_st.st_mode) &&
+			    (dir_st.st_mode & S_ISGID))
+				st.st_gid = dir_st.st_gid;
+			else
+				st.st_gid = getgid ();
+		}
+#endif
+	}
+
+	/* Save to a temporary file.  We set the umask because some (buggy)
+	 * implementations of mkstemp() use permissions 0666 and we want 0600.
+	 */
+	temp_filename = g_build_filename (dirname, ".gsf-save-XXXXXX", NULL);
+	/* Oh, joy.  What about threads?  --MW */
+	saved_umask = umask (0077);
+	fd = g_mkstemp (temp_filename); /* this modifies temp_filename to the used name */
+	umask (saved_umask);
+
+	if (fixup_mode)
+		st.st_mode &= ~saved_umask;
+
+	if (fd < 0 || NULL == (file = fdopen (fd, "wb"))) {
+		if (err != NULL) {
+			int save_errno = errno;
+			char *dname = g_filename_display_name
+				(temp_filename);
+			*err = g_error_new
+				(gsf_output_error_id (), errno,
+				 "%s: %s",
+				 dname, g_strerror (save_errno));
+			g_free (dname);
+		}
+		goto failure;
+	}
+
+	stdio = (GsfOutputStdio *)g_object_new_valist (GSF_OUTPUT_STDIO_TYPE,
+		first_property_name, var_args);
+	stdio->file = file;
+	stdio->st = st;
+	stdio->create_backup_copy = FALSE;
+	stdio->real_filename = real_filename;
+	stdio->temp_filename = temp_filename;
+
+	gsf_output_set_name_from_filename (GSF_OUTPUT (stdio), filename);
+
+	g_free (dirname);
+
+	return GSF_OUTPUT (stdio);
+
+ failure:
+	g_free (temp_filename);
+	g_free (real_filename);
+	g_free (dirname);
+	return NULL;
+}
+
+/**
+ * gsf_output_stdio_new_full :
+ * @filename : in utf8.
+ * @err	     : optionally NULL.
+ * @first_property_name : NULL terminated list of properties
+ * @Varargs : 
+ *
+ * Returns a new file or NULL.
+ **/
+GsfOutput *
+gsf_output_stdio_new_full (char const *filename, GError **err,
+			   char const *first_property_name, ...)
+{
+	GsfOutput *res;
+	va_list var_args;
+	
+	va_start (var_args, first_property_name);
+	res = gsf_output_stdio_new_valist (filename, err, first_property_name, var_args);
+	va_end (var_args);
+
+	return res;
+}
+
 /**
  * gsf_output_stdio_new :
  * @filename : in utf8.
- * @file     : an existing stdio FILE *
+ * @err	     : optionally NULL.
+ *
+ * Returns a new file or NULL.
+ **/
+GsfOutput *
+gsf_output_stdio_new (char const *filename, GError **err)
+{
+	return gsf_output_stdio_new_full (filename, err, NULL);
+}
+
+/**
+ * gsf_output_stdio_new_FILE :
+ * @filename  : The filename corresponding to @file.
+ * @file      : an existing stdio FILE *
  * @keep_open : Should @file be closed when the wrapper is closed
  *
- * Assumes ownership of @file.
+ * Assumes ownership of @file.  If @keep_open is true, ownership reverts
+ * to caller when the Gsfobject is closed.
  *
- * Returns a new GsfOutput wrapper for @file
+ * Returns a new GsfOutput wrapper for @file.  Warning: the result will be
+ * seekable only if @file is seekable.  If it is seekable, the resulting
+ * GsfOutput object will seek relative to @file's beginning, not its
+ * current location at the time the GsfOutput object is created.
  **/
 GsfOutput *
 gsf_output_stdio_new_FILE (char const *filename, FILE *file, gboolean keep_open)
