@@ -63,13 +63,11 @@ blob_finalize (GObject *obj)
 }
 
 static GsfInput *
-blob_dup (GsfInput *input, GError **err)
+blob_dup (GsfInput *input, G_GNUC_UNUSED GError **err)
 {
 	GsfStructuredBlob const *src = (GsfStructuredBlob *) input;
 	GsfStructuredBlob *dst = g_object_new (GSF_STRUCTURED_BLOB_TYPE, NULL);
 
-	(void)err;
-
 	if (src->data != NULL) {
 		dst->data = src->data;
 		g_object_ref (G_OBJECT (dst->data));
