@@ -28,7 +28,8 @@
 
 G_BEGIN_DECLS
 
-#define GSF_TIMESTAMP_TYPE        (gsf_timestamp_get_type ())
+#define GSF_TIMESTAMP_TYPE      (gsf_timestamp_get_type ())
+#define VAL_IS_GSF_TIMESTAMP(v) (G_TYPE_CHECK_VALUE_TYPE((v), GSF_TIMESTAMP_TYPE))
 
 struct _GsfTimestamp {
 	GDate	  date;			/* In local timezone */
@@ -52,7 +53,7 @@ guint         gsf_timestamp_hash	(GsfTimestamp const *stamp);
 gboolean      gsf_timestamp_equal	(GsfTimestamp const *a,
 					 GsfTimestamp const *b);
 
-void g_value_set_timestamp (GValue *value, GsfTimestamp const *stamp);
+void gsf_value_set_timestamp (GValue *value, GsfTimestamp const *stamp);
 
 G_END_DECLS
 
