@@ -22,6 +22,7 @@
 #include <gsf-config.h>
 #include <gsf/gsf-utils.h>
 #include <gsf/gsf-input.h>
+#include <gobject/gvaluecollector.h>
 
 #include <ctype.h>
 #include <stdio.h>
@@ -63,8 +64,18 @@ gsf_shutdown (void)
 {
 }
 
+void
+gsf_init_dynamic (GTypeModule *module)
+{
+}
+
+void
+gsf_shutdown_dynamic (GTypeModule *module)
+{
+}
+
 static void
-gsf_mem_dump_full (guint8 const *ptr, size_t len, unsigned offset)
+gsf_mem_dump_full (guint8 const *ptr, size_t len, guint offset)
 {
 	size_t i, j, off;
 
@@ -234,7 +245,7 @@ gsf_le_get_double (void const *p)
 #if G_FLOAT_BYTE_ORDER == G_ARMFLOAT_ENDIAN
 	double data;
 	memcpy ((char *)&data + 4, p, 4);
-	memcpy ((char *)&data, (const char *)p + 4, 4);
+	memcpy ((char *)&data, (char const *)p + 4, 4);
 	return data;
 #elif G_FLOAT_BYTE_ORDER == G_BIG_ENDIAN
 	if (sizeof (double) == 8) {
@@ -272,7 +283,7 @@ void
 gsf_le_set_double (void *p, double d)
 {
 #if G_FLOAT_BYTE_ORDER == G_ARMFLOAT_ENDIAN
-	memcpy (p, (const char *)&d + 4, 4);
+	memcpy (p, (char const *)&d + 4, 4);
 	memcpy ((char *)p + 4, &d, 4);
 #elif G_FLOAT_BYTE_ORDER == G_BIG_ENDIAN
 	if (sizeof (double) == 8) {
@@ -315,15 +326,20 @@ gsf_le_set_double (void *p, double d)
 char const *
 gsf_extension_pointer (char const *path)
 {
-	char *s, *t;
+	char const *s, *end;
 	
 	g_return_val_if_fail (path != NULL, NULL);
 
-	t = strrchr (path, G_DIR_SEPARATOR);
-	s = strrchr ((t != NULL) ? t : path, '.');
-	if (s != NULL)
-		return s + 1;
-	return path + strlen(path);
+	end = path + strlen (path);
+	for (s = end; s > path; ) {
+		s--;
+		if (G_IS_DIR_SEPARATOR (*s))
+			break;
+		if (*s == '.')
+			return s + 1;
+	}
+
+	return end;
 }
 
 /**
@@ -337,11 +353,6 @@ gsf_iconv_close (GIConv handle)
 		g_iconv_close (handle);
 }
 
-/* FIXME: what about translations?  */
-#ifndef _
-#define _(x) x
-#endif
-
 /**
  * gsf_filename_to_utf8:
  * @filename: file name suitable for open(2).
@@ -356,32 +367,16 @@ gsf_iconv_close (GIConv handle)
 char *
 gsf_filename_to_utf8 (char const *filename, gboolean quoted)
 {
-	GError *err = NULL;
-	char *res = g_filename_to_utf8 (filename, -1, NULL, NULL, &err);
-
-	if (err) {
-		char *msg;
-		if (res && res[0])
-			/*
-			 * Translators: the ellipsis ("...") here refers to
-			 * a truncated file name.
-			 */
-			msg = g_strdup_printf (_("(Invalid file name: \"%s...\")"),
-					       res);
-		else
-			msg = g_strdup (_("(Invalid file name)"));
-		g_free (res);
-		g_clear_error (&err);
-		return msg;
-	}
+	char *dname = g_filename_display_name (filename);
+	char *result;
 
 	if (quoted) {
-		char *newres = g_strdup_printf (_("\"%s\""), res);
-		g_free (res);
-		res = newres;
-	}						
+		result = g_strconcat ("\"", dname, "\"", NULL);
+		g_free (dname);
+	} else
+		result = dname;
 
-	return res;
+	return result;
 }
 
 /***************************************************************************/
@@ -571,7 +566,7 @@ gsf_base64_encode_step (guint8 const *in, size_t len,
  **/
 size_t
 gsf_base64_decode_step (guint8 const *in, size_t len, guint8 *out,
-			int *state, unsigned int *save)
+			int *state, guint *save)
 {
 	register guint8 const *inptr;
 	register guint8 *outptr, c;
@@ -644,3 +639,99 @@ gsf_base64_decode_simple (guint8 *data, size_t len)
 	unsigned int save = 0;
 	return gsf_base64_decode_step (data, len, data, &state, &save);
 }
+
+
+/* Largely a copy of g_object_new_valist.  */
+/**
+ * gsf_property_settings_collect_valist: collect property setting from a va_list.
+ * @object_type: the GType for which the properties are being set.
+ * @p_n_params: a pointer to the number of properties collected.  (Used for
+ *   both input and output.)
+ * @p_params: a pointer to the GParameter array that holds the properties.
+ *   (Used for both input and output.  This may point to a NULL pointer if
+ *   there are no properties collected yet.)
+ * @first_property_name: the name of the first property being set, or NULL.
+ * @var_args: a va_list holding the remainder of the property names and
+ *   values, terminated by a NULL.
+ *
+ * This function builds a GParameter array suitable for g_object_newv.
+ **/
+void
+gsf_property_settings_collect_valist (GType object_type,
+				      GParameter **p_params,
+				      size_t *p_n_params,
+				      const gchar *first_property_name,
+				      va_list var_args)
+{
+  GObjectClass *class;
+  GParameter *params = *p_params;
+  const gchar *name;
+  size_t n_params = *p_n_params;
+  size_t n_alloced_params = n_params;  /* We might have more.  */
+
+  g_return_if_fail (G_TYPE_IS_OBJECT (object_type));
+
+  class = g_type_class_ref (object_type);
+
+  name = first_property_name;
+  while (name)
+    {
+      gchar *error = NULL;
+      GParamSpec *pspec = g_object_class_find_property (class, name);
+      if (!pspec)
+	{
+	  g_warning ("%s: object class `%s' has no property named `%s'",
+		     G_STRFUNC,
+		     g_type_name (object_type),
+		     name);
+	  break;
+	}
+
+      if (n_params >= n_alloced_params)
+	{
+	  n_alloced_params += 16;
+	  params = g_renew (GParameter, params, n_alloced_params);
+	}
+      params[n_params].name = name;
+      params[n_params].value.g_type = 0;
+      g_value_init (&params[n_params].value, G_PARAM_SPEC_VALUE_TYPE (pspec));
+      G_VALUE_COLLECT (&params[n_params].value, var_args, 0, &error);
+      if (error)
+	{
+	  g_warning ("%s: %s", G_STRFUNC, error);
+	  g_free (error);
+          g_value_unset (&params[n_params].value);
+	  break;
+	}
+      n_params++;
+      name = va_arg (var_args, gchar*);
+    }
+
+  g_type_class_unref (class);
+
+  *p_params = params;
+  *p_n_params = n_params;
+}
+
+/* This is a vararg version of gsf_property_settings_collect_valist.  */
+void
+gsf_property_settings_collect (GType object_type,
+			       GParameter **p_params,
+			       size_t *p_n_params,
+			       const gchar *first_property_name,
+			       ...)
+{
+  va_list var_args;
+  va_start (var_args, first_property_name);
+  gsf_property_settings_collect_valist (object_type, p_params, p_n_params, first_property_name, var_args);
+  va_end (var_args);
+}
+
+void
+gsf_property_settings_free (GParameter *params,
+			    size_t n_params)
+{
+	while (n_params--)
+		g_value_unset (&params[n_params].value);
+	g_free (params);
+}
